@@ -66,25 +66,20 @@ def includeme(config: Configurator) -> None:
 
 def openapi_validated(request: Request) -> dict:
     """Get validated parameters."""
-    # Validate request and attach all findings for view to introspect
-    validate_request = asbool(
-        request.registry.settings.get(
-            "pyramid_openapi3.enable_request_validation", True
+
+    # We need this here in case someone calls request.openapi_validated on
+    # a view marked with openapi=False
+    if not request.environ.get("pyramid_openapi3.enabled"):
+        raise AttributeError(
+            "Cannot do openapi request validation on a view marked with openapi=False"
         )
-    )
-    validate_response = asbool(
-        request.registry.settings.get(
-            "pyramid_openapi3.enable_response_validation", True
-        )
-    )
-    request.environ["pyramid_openapi3.validate_response"] = validate_response
+
     gsettings = settings = request.registry.settings["pyramid_openapi3"]
     route_settings = gsettings.get("routes")
     if route_settings and request.matched_route.name in route_settings:
         settings = request.registry.settings[route_settings[request.matched_route.name]]
 
-    if validate_request:  # pragma: no branch
-        request.environ["pyramid_openapi3.validate_request"] = True
+    if request.environ.get("pyramid_openapi3.validate_request"):
         openapi_request = PyramidOpenAPIRequestFactory.create(request)
         validated = settings["request_validator"].validate(openapi_request)
         return validated
@@ -109,12 +104,29 @@ def openapi_view(view: View, info: ViewDeriverInfo) -> View:
     if info.options.get("openapi"):
 
         def wrapper_view(context: Context, request: Request) -> Response:
-            validate_request = asbool(
+
+            # We need this to be able to raise AttributeError if view code
+            # accesses request.openapi_validated on a view that is marked
+            # with openapi=False
+            request.environ["pyramid_openapi3.enabled"] = True
+
+            # If view is marked with openapi=True (i.e. we are in this
+            # function) and registry settings are not set to disable
+            # validation, then do request/response validation
+            request.environ["pyramid_openapi3.validate_request"] = asbool(
                 request.registry.settings.get(
                     "pyramid_openapi3.enable_request_validation", True
                 )
             )
-            if validate_request and request.openapi_validated.errors:
+            request.environ["pyramid_openapi3.validate_response"] = asbool(
+                request.registry.settings.get(
+                    "pyramid_openapi3.enable_response_validation", True
+                )
+            )
+
+            # Request validation can happen already here, but response validation
+            # needs to happen later in a tween
+            if request.openapi_validated and request.openapi_validated.errors:
                 raise RequestValidationError(errors=request.openapi_validated.errors)
 
             # Do the view
